#ifndef BINDINGS_H
#define BINDINGS_H

#include <X11/Xlib.h>

typedef enum {
  PointerButtonPress,
  PointerMotion
} PointerEventType;

typedef struct {
  PointerEventType type;
  unsigned int time;
  int x;
  int y;
  unsigned int modifier_mask; 
} pointer_event_t;

typedef struct {
  char *str;
  void (*fun)(void *);
} action_mapping_t;


void actionUpperButtonSelect(void *data);
void actionLowerButtonSelect(void *data);
void actionLeftButtonSelect(void *data);
void actionRightButtonSelect(void *data);
void actionButtonActivate(void *data);
void actionMenuCallTitle(void *data);
void actionMenuCallRoot(void *data);
void actionMenuCallSubpicture(void *data);
void actionMenuCallAudio(void *data);
void actionMenuCallAngle(void *data);
void actionMenuCallPTT(void *data);
void actionResume(void *data);
void actionPauseToggle(void *data);
void actionPauseOn(void *data);
void actionPauseOff(void *data);
void actionSubpictureToggle(void *data);
void actionNextPG(void *data);
void actionPrevPG(void *data);
void actionQuit(void *data);
void actionFullScreenToggle(void *data);
void actionForwardScan(void *data);
void actionPlay(void *data);
void actionFastForward(void *data);
void actionSlowForward(void *data);
void actionFaster(void *data);
void actionSlower(void *data);

static action_mapping_t actions[] = {
  { "Play", actionPlay },
  { "PauseToggle", actionPauseToggle },
  { "Stop", NULL },
  { "FastForward", actionFastForward },
  { "SlowForward", actionSlowForward },
  { "Faster", actionFaster },
  { "Slower", actionSlower },
  { "NextPG", actionNextPG },
  { "PrevPG", actionPrevPG },
  { "UpperButton", actionUpperButtonSelect },
  { "LowerButton", actionLowerButtonSelect },
  { "LeftButton", actionLeftButtonSelect },
  { "RightButton", actionRightButtonSelect},
  { "ButtonActivate", actionButtonActivate },
  { "TitleMenu", actionMenuCallTitle },
  { "RootMenu", actionMenuCallRoot },
  { "AudioMenu", actionMenuCallAudio },
  { "AngleMenu", actionMenuCallAngle },
  { "PTTMenu", actionMenuCallPTT },
  { "SubtitleMenu", actionMenuCallSubpicture },
  { "Resume", actionResume },
  { "FullScreenToggle", actionFullScreenToggle },
  { "SubtitleToggle", actionSubpictureToggle },
  { "Quit", actionQuit },

  { NULL, NULL }
};

#endif /* BINDINGS_H */
