/* Ogle - A video player
 * IR Control Client (chres.c)
 * Copyright (C) 2001 Rossen Tchobanski <rosko@rosko.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#define	PROGNAME	"chres"


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/extensions/xf86vmode.h>
#include <X11/extensions/xf86dga.h>


int getVideoMode(int *w, int *h)
{
  char *disname = NULL;
  Display *dpy = XOpenDisplay(disname);
  return XF86DGAGetViewPortSize(dpy,DefaultScreen(dpy), w, h);
}

int setVideoMode(int w, int h)
{
  int i=0;
  int ratio=0;
  int winnerRatio=32000;
  Display *dpy;
  int prevW, prevH;
  int nmodes;
  XF86VidModeModeInfo **modes;
  XF86VidModeModeInfo *selected=0L;
  char *disname = NULL;

  int  foo,bar;
#ifdef DEBUG
  int  ma,mi;
#endif

  if(!(dpy = XOpenDisplay(disname)))  {
    fprintf(stderr, "%s:  Unable to open display %s\n",
    	    (disname != NULL) ? disname : XDisplayName(NULL), PROGNAME);
    exit(-1);
  }
   
  if(!XF86VidModeQueryExtension(dpy,&foo,&bar)) {
    fprintf(stderr, "%s: no VideoModeExtension\n", PROGNAME);
    exit(-1);
  }
   
#ifdef DEBUG
  XF86VidModeQueryVersion(dpy,&ma,&mi);
  fprintf(stderr, "%s: VideoModeExtension Version is %d.%d\n", PROGNAME, ma, mi);
  fprintf(stderr, "%s: display '%s', dpy '%p'\n", PROGNAME, disname, dpy);
#endif

  /* Get all available video modes */
  XF86VidModeGetAllModeLines( dpy, XDefaultScreen(dpy), &nmodes, &modes);

#ifdef DEBUG
  for (i = 0; i < nmodes; i++)
    fprintf(stderr, "%s: %dx%d\n",PROGNAME,modes[i]->hdisplay,modes[i]->vdisplay);
#endif

  /* Save the original video mode settings */
  XF86DGAGetViewPortSize(dpy,DefaultScreen(dpy), &prevW, &prevH);
  printf("%s: original video mode %dx%d\n",PROGNAME,prevW,prevH);

  /* Search for the closest video mode */
  for ( i=0; i<nmodes; ++i ) {
    /* first check if the video mode fits the requested size */
    if ((modes[i]->hdisplay >= w) && (modes[i]->vdisplay >= h)) {
      /* calculates a near-ratio to find the closest vm */
      ratio=(modes[i]->hdisplay - w) + (modes[i]->vdisplay - h); {
         winnerRatio=ratio;
         selected=modes[i];
      }
    }
  }

  if ( selected!=0L ) {
    int screen = DefaultScreen(dpy);
    int ret;
    
    if ((selected->hdisplay!=prevW) || (selected->vdisplay!=prevH)) {
       /* Switch to the selected video mode */
       XF86VidModeLockModeSwitch(dpy, screen, 0 );
       ret = XF86VidModeValidateModeLine(dpy, screen, selected);
       
       
       if(XF86VidModeSwitchToMode(dpy, screen, selected)) {
         printf("Switching to mode %dx%d (screen=%d, status=%d)\n",
	    selected->hdisplay, selected->vdisplay,
	    screen, ret);
    	 XF86VidModeSetViewPort(dpy, screen, 0, 0);
	 XSync(dpy, 0);
         XCloseDisplay(dpy);
       } else {
    	 printf("Couldn't find an appropriate mode %dx%d,\n",selected->hdisplay,selected->vdisplay);
       }
    }
    return 1;
  } else
    return 0;
}
