/* Ogle - A video player
 * IR Control Client
 * Copyright (C) 2001 Rossen Tchobanski <rosko@rosko.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <sys/msg.h>
#include <unistd.h>
#include <stdlib.h>

#include <ogle/dvdcontrol.h>
#include "bindings.h"

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include <lirc/lirc_client.h>

#include <sys/socket.h>
#include <sys/un.h>
#include <errno.h>

#define VERSION "0.0.1"

char *program_name;
int msgqid;

DVDNav_t *nav;
char *dvd_path;

int lirc_fd=-1;
struct lirc_config *config=NULL;

extern int setVideoMode(int w, int h);
extern int getVideoMode(int *w, int *h);


void usage()
{
  fprintf(stderr, "Usage: %s -m <msgid> [-v] [-h]\n", 
          program_name);
}

void version()
{
  fprintf(stderr, "%s version %s\n", 
          program_name, VERSION);
}

void cleanup()
{
  if(lirc_fd!=-1) {
  	  lirc_deinit();
  	  lirc_fd=-1;
  }
}

void init()
{
  DVDResult_t res;
  
  if((lirc_fd=lirc_init("ogle",1))==-1) {
  	  fprintf(stderr,"%s: could not init LIRC support\n",
  		  program_name);
  	  return;
  }
  if(lirc_readconfig(NULL,&config,NULL)==-1) {
  	  lirc_deinit();
  	  fprintf(stderr,
  		  "%s: could not read LIRC config file\n"
  		  "%s: please read the documentation of LIRC \n"
  		  "%s: how to create a proper config file\n",
  		  program_name,program_name,program_name);
  	  return;
  }
  
  if(msgqid !=-1) { // ignore sending data.
    sleep(1);
    res = DVDOpenNav(&nav, msgqid);
    if(res != DVD_E_Ok ) {
      DVDPerror("DVDOpen:", res);
      exit(1);
    }
  }
}


int main(int argc, char *argv[])
{
  int c,n;
  int ret;	
  char *action;
  char *code; 
  int fullScreen = 0;
  int w, h;
    
  program_name = argv[0];

  /* Parse command line options */
  while ((c = getopt(argc, argv, "m:h?v")) != EOF) {
    switch (c) {
    case 'm':
      msgqid = atoi(optarg);
      break;
    case 'v':
      version();
      exit(0);
    case 'h':
    case '?':
      usage();
      return 1;
    }
  }
  
  if((argc - optind > 1) || (argc == 1)) {
    usage();
    exit(1);
  }

  init();
  
  getVideoMode(&w, &h);

  while((ret=lirc_nextcode(&code))==0 && code!=NULL) {
    while((ret=lirc_code2char(config,code,&action))==0 && action!=NULL) {
//      fprintf(stderr,"action %s\n",action);
      if (!strcmp("VOL_UP", action))
        system("aumix -w +5");
      if (!strcmp("VOL_DOWN", action))
	system("aumix -w -5");
      if (!strcmp("FullScreenToggle", action)) {
	if(!fullScreen) {
	  setVideoMode(800,600);
	  fullScreen = 1;
	} else {
	  setVideoMode(w, h);
	  fullScreen = 0;
	}   
      }	
      for(n = 0; actions[n].str != NULL; n++) {
        if(!strcmp(actions[n].str, action)) {
          if(actions[n].fun != NULL)  actions[n].fun(NULL);
	  continue;
	}
      }	
    }
    free(code);
    if(ret==-1) break;
  }
  if(ret==-1) {
    /* something went badly wrong */
    fprintf(stderr,"%s: disconnected from LIRC\n",program_name);
    cleanup();
    exit(1);
  }

  return 0;
}
